#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noipy
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and epl-v10.html for details.

"""
A simple script to update IP on No-IP DNS service. 
"""
__title__ = 'noipy'
__version__ = '1.2.1'
__author__ = 'Pablo O Vieira'
__email__ = 'email@povieira.com'
__license__ = 'Eclipse Public License (EPL) - v1.0'

